use std::time::Duration;

use sdl2::{
    event::Event, keyboard::Keycode, pixels::Color, render::Canvas, video::Window, Sdl,
    VideoSubsystem,
};

pub struct Game {
    sdl_context: Sdl,
    video_subsystem: VideoSubsystem,
    canvas: Canvas<Window>,
    event_pump: sdl2::EventPump,
}

#[derive(Debug, Clone)]
pub struct Error(String);

impl std::fmt::Display for Error {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}", self.0)
    }
}

impl<T: std::fmt::Display> From<T> for Error {
    fn from(value: T) -> Self {
        Self(value.to_string())
    }
}

impl Game {
    pub fn new() -> Result<Self, Error> {
        let sdl_context = sdl2::init()?;
        let video_subsystem = sdl_context.video()?;
        let window = video_subsystem
            .window("pvp-game-dilapidation", 1280, 720)
            .position_centered()
            .build()?;
        let mut canvas = window.into_canvas().build()?;
        canvas.set_draw_color(Color::RGB(60, 180, 180));
        canvas.clear();
        let event_pump = sdl_context.event_pump()?;
        Ok(Self {
            sdl_context,
            video_subsystem,
            canvas,
            event_pump,
        })
    }

    pub fn run(mut self) {
        'running: loop {
            for event in self.event_pump.poll_iter() {
                match event {
                    Event::Quit { .. }
                    | Event::KeyDown {
                        keycode: Some(Keycode::Escape),
                        ..
                    } => break 'running,
                    _ => {}
                }
            }
            self.canvas.present();
        }
        self.canvas.present();
        std::thread::sleep(Duration::new(0, 1_000_000_000u32 / 60))
    }
}
